# Load the data
nlsy_math_wide<-read.table("C:/GRE2016/Data/nlsy_math_hyp_wide_R.dat", na.string='.')

# Give the variable names
names(nlsy_math_wide)<-c('id','female','lb_wght','anti_k1',
                         'math2','math3','math4','math5','math6','math7','math8',
                         'comp2','comp3','comp4','comp5','comp6','comp7','comp8',
                         'rec2','rec3','rec4','rec5','rec6','rec7','rec8',
                         'bpi2','bpi3','bpi4','bpi5','bpi6','bpi7','bpi8',
                         'asl2','asl3','asl4','asl5','asl6','asl7','asl8',
                         'ax2','ax3','ax4','ax5','ax6','ax7','ax8',
                         'hds2','hds3','hds4','hds5','hds6','hds7','hds8',
                         'hyp2','hyp3','hyp4','hyp5','hyp6','hyp7','hyp8',
                         'dpn2','dpn3','dpn4','dpn5','dpn6','dpn7','dpn8',
                         'wdn2','wdn3','wdn4','wdn5','wdn6','wdn7','wdn8',
                         'age2','age3','age4','age5','age6','age7','age8',
                         'men2','men3','men4','men5','men6','men7','men8',
                         'spring2','spring3','spring4','spring5','spring6','spring7','spring8',
                         'anti2','anti3','anti4','anti5','anti6','anti7','anti8')

library(OpenMx)


#### No Coupling ####
dcm.no.omx <- mxModel("No Coupling Dual Change Model, Path Specification",
 type="RAM", mxData(observed=nlsy_math_wide, type="raw" ),
 manifestVars=c('math2','math3','math4','math5','math6','math7','math8',
                'rec2','rec3','rec4','rec5','rec6','rec7','rec8'),
 latentVars=c('lm1','lm2','lm3','lm4','lm5','lm6','lm7',
                    'dm2','dm3','dm4','dm5','dm6','dm7',
                    'g2',
              'lr1','lr2','lr3','lr4','lr5','lr6','lr7',
                    'dr2','dr3','dr4','dr5','dr6','dr7',
                    'j2'),

# defining true scores
mxPath(from=c('lm1','lm2','lm3','lm4','lm5','lm6','lm7',
              'lr1','lr2','lr3','lr4','lr5','lr6','lr7'),
 to=c('math2','math3','math4','math5','math6','math7','math8',
     'rec2','rec3','rec4','rec5','rec6','rec7','rec8'),
 arrows=1, free=FALSE, values=1),

# Autoregressive Effects
mxPath(from=c('lm1','lm2','lm3','lm4','lm5','lm6',
              'lr1','lr2','lr3','lr4','lr5','lr6'),
 to=c('lm2','lm3','lm4','lm5','lm6','lm7',
      'lr2','lr3','lr4','lr5','lr6','lr7'),
 arrows=1, free=FALSE, values=1),

# defining change scores
mxPath(from=c('dm2','dm3','dm4','dm5','dm6','dm7',
              'dr2','dr3','dr4','dr5','dr6','dr7'),
 to=c('lm2','lm3','lm4','lm5','lm6','lm7',
      'lr2','lr3','lr4','lr5','lr6','lr7'),
 arrows=1, free=FALSE, values=1),

# defining constant change factor for mathematics 
mxPath(from='g2', to=c('dm2','dm3','dm4','dm5','dm6','dm7'),
 arrows=1, free=FALSE, values=1),

# defining constant change factor for reading 
mxPath(from='j2', to=c('dr2','dr3','dr4','dr5','dr6','dr7'),
 arrows=1, free=FALSE, values=1),

# unique variance paths
mxPath(from=c('math2','math3','math4','math5','math6','math7','math8'),
 arrows=2, free=TRUE, values=35, labels='sigma2_u'),

mxPath(from=c('rec2','rec3','rec4','rec5','rec6','rec7','rec8'),
 arrows=2, free=TRUE, values=35, labels='sigma2_s'),

# unique covariance paths
mxPath(from=c('math2','math3','math4','math5','math6','math7','math8'),
 to=c('rec2','rec3','rec4','rec5','rec6','rec7','rec8'),
 arrows=2, free=TRUE, values=0, labels='sigma_su'),

# latent variable variances and covariances 
mxPath(from=c('lm1','g2','lr1','j2'), 
 arrows=2, connect = 'unique.pairs', free=TRUE, 
 values = c(75, 0, 0, 0, 7, 0, 0, 75, 0, 10),
 labels=c('sigma2_lm1','sigma_g2lm1','sigma_lr1lm1','sigma_j2lm1',
                       'sigma2_g2'  ,'sigma_lr1g2','sigma_j2g2',
                                     'sigma2_lr1' ,'sigma_j2lr1',
                                                   'sigma2_j2')),

# proportional parameters 
mxPath(from=c('lm1','lm2','lm3','lm4','lm5','lm6'),
 to=c('dm2','dm3','dm4','dm5','dm6','dm7'),
 arrows=1, free=TRUE, values=-.2, labels='pi_m'),

mxPath(from=c('lr1','lr2','lr3','lr4','lr5','lr6'),
 to=c('dr2','dr3','dr4','dr5','dr6','dr7'),
 arrows=1, free=TRUE, values=-.2, labels='pi_r'),

# means and intercepts 
mxPath(from='one', to=c('lm1','g2','lr1','j2'),
 arrows=1, free=TRUE, values=c(30, 15, 30, 15),
 labels=c('gamma_lm1','gamma_g2','gamma_lr1','gamma_j2'))

) # close model

dcm.no.fit <- mxRun(dcm.no.omx)

summary(dcm.no.fit) 

#Summary of No Coupling Dual Change Model, Path Specification 
# 
#free parameters:
#           name matrix   row   col   Estimate  Std.Error A
#1          pi_m      A   dm2   lm1 -0.2462091 0.01773480  
#2          pi_r      A   dr2   lr1 -0.1550602 0.01629289  
#3      sigma2_u      S math2 math2 31.1002767 1.70561620  
#4      sigma_su      S math2  rec2  6.8716781 1.27677775  
#5      sigma2_s      S  rec2  rec2 33.6625887 1.82395716  
#6    sigma2_lm1      S   lm1   lm1 71.5990792 6.51494782  
#7   sigma_g2lm1      S   lm1    g2 14.2719931 1.67751209  
#8     sigma2_g2      S    g2    g2  5.7047257 0.83618942  
#9  sigma_lr1lm1      S   lm1   lr1 58.2268165 5.35006315  
#10  sigma_lr1g2      S    g2   lr1 12.7295955 1.61812795  
#11   sigma2_lr1      S   lr1   lr1 84.5798023 6.93652842  
#12  sigma_j2lm1      S   lm1    j2 10.5088942 1.64166309  
#13   sigma_j2g2      S    g2    j2  3.7759940 0.57977120  
#14  sigma_j2lr1      S   lr1    j2 18.1052856 1.93858405  
#15    sigma2_j2      S    j2    j2  5.5025180 0.95285443  
#16    gamma_lm1      M     1   lm1 32.4316474 0.43219531  
#17     gamma_g2      M     1    g2 15.4608641 0.80252833  
#18    gamma_lr1      M     1   lr1 34.0996017 0.43851143  
#19     gamma_j2      M     1    j2 12.4389779 0.77376412  
#
#observed statistics:  4435 
#estimated parameters:  19 
#degrees of freedom:  4416 
#fit value ( -2lnL units ):  31376.47 
#number of observations:  933 
#Information Criteria: 
#      |  df Penalty  |  Parameters Penalty  |  Sample-Size Adjusted
#AIC:      22544.470               31414.47                       NA
#BIC:       1178.073               31506.40                 31446.06
#Some of your fit indices are missing.
#  To get them, fit saturated and independence models, and include them with
#  summary(yourModel, refModels=...) 
#  See help(mxRefModels) for an easy way of doing this in many cases. 
#timestamp: 2016-04-18 14:39:54 
#Wall clock time (HH:MM:SS.hh): 00:00:04.05 
#OpenMx version number: 2.2.4 
#Need help?  See help(mxSummary) 

#### Mathematics -> Changes in Reading ####
dcm.mr.omx <- mxModel("Math to Reading Dual Change Model, Path Specification",
 type="RAM", mxData(observed=nlsy_math_wide, type="raw" ),
 manifestVars=c('math2','math3','math4','math5','math6','math7','math8',
                'rec2','rec3','rec4','rec5','rec6','rec7','rec8'),
 latentVars=c('lm1','lm2','lm3','lm4','lm5','lm6','lm7',
                    'dm2','dm3','dm4','dm5','dm6','dm7',
                    'g2',
              'lr1','lr2','lr3','lr4','lr5','lr6','lr7',
                    'dr2','dr3','dr4','dr5','dr6','dr7',
                    'j2'),

# defining true scores
mxPath(from=c('lm1','lm2','lm3','lm4','lm5','lm6','lm7',
              'lr1','lr2','lr3','lr4','lr5','lr6','lr7'),
 to=c('math2','math3','math4','math5','math6','math7','math8',
     'rec2','rec3','rec4','rec5','rec6','rec7','rec8'),
 arrows=1, free=FALSE, values=1),

# Autoregressive Effects
mxPath(from=c('lm1','lm2','lm3','lm4','lm5','lm6',
              'lr1','lr2','lr3','lr4','lr5','lr6'),
 to=c('lm2','lm3','lm4','lm5','lm6','lm7',
      'lr2','lr3','lr4','lr5','lr6','lr7'),
 arrows=1, free=FALSE, values=1),

# defining change scores
mxPath(from=c('dm2','dm3','dm4','dm5','dm6','dm7',
              'dr2','dr3','dr4','dr5','dr6','dr7'),
 to=c('lm2','lm3','lm4','lm5','lm6','lm7',
      'lr2','lr3','lr4','lr5','lr6','lr7'),
 arrows=1, free=FALSE, values=1),

# defining constant change factor for mathematics 
mxPath(from='g2', to=c('dm2','dm3','dm4','dm5','dm6','dm7'),
 arrows=1, free=FALSE, values=1),

# defining constant change factor for reading 
mxPath(from='j2', to=c('dr2','dr3','dr4','dr5','dr6','dr7'),
 arrows=1, free=FALSE, values=1),

# unique variance paths
mxPath(from=c('math2','math3','math4','math5','math6','math7','math8'),
 arrows=2, free=TRUE, values=35, labels='sigma2_u'),

mxPath(from=c('rec2','rec3','rec4','rec5','rec6','rec7','rec8'),
 arrows=2, free=TRUE, values=35, labels='sigma2_s'),

# unique covariance paths
mxPath(from=c('math2','math3','math4','math5','math6','math7','math8'),
 to=c('rec2','rec3','rec4','rec5','rec6','rec7','rec8'),
 arrows=2, free=TRUE, values=0, labels='sigma_su'),

# latent variable variances and covariances 
mxPath(from=c('lm1','g2','lr1','j2'), 
 arrows=2, connect = 'unique.pairs', free=TRUE, 
 values = c(75, 0, 0, 0, 7, 0, 0, 75, 0, 10),
 labels=c('sigma2_lm1','sigma_g2lm1','sigma_lr1lm1','sigma_j2lm1',
                       'sigma2_g2'  ,'sigma_lr1g2','sigma_j2g2',
                                     'sigma2_lr1' ,'sigma_j2lr1',
                                                   'sigma2_j2')),

# proportional parameters 
mxPath(from=c('lm1','lm2','lm3','lm4','lm5','lm6'),
 to=c('dm2','dm3','dm4','dm5','dm6','dm7'),
 arrows=1, free=TRUE, values=-.2, labels='pi_m'),

mxPath(from=c('lr1','lr2','lr3','lr4','lr5','lr6'),
 to=c('dr2','dr3','dr4','dr5','dr6','dr7'),
 arrows=1, free=TRUE, values=-.2, labels='pi_r'),

# coupling parameters 
mxPath(from=c('lm1','lm2','lm3','lm4','lm5','lm6'),
 to=c('dr2','dr3','dr4','dr5','dr6','dr7'),
 arrows=1, free=TRUE, values=0, labels='delta_r'),

# means and intercepts 
mxPath(from='one', to=c('lm1','g2','lr1','j2'),
 arrows=1, free=TRUE, values=c(30, 15, 30, 15),
 labels=c('gamma_lm1','gamma_g2','gamma_lr1','gamma_j2'))

) # close model

dcm.mr.fit <- mxRun(dcm.mr.omx)

summary(dcm.mr.fit) 

#Summary of Math to Reading Dual Change Model, Path Specification 
# 
#free parameters:
#           name matrix   row   col   Estimate  Std.Error A
#1          pi_m      A   dm2   lm1 -0.2347373 0.01783359  
#2       delta_r      A   dr2   lm1  0.4169444 0.12503785  
#3          pi_r      A   dr2   lr1 -0.5195133 0.10955141  
#4      sigma2_u      S math2 math2 31.4229083 1.64365740  
#5      sigma_su      S math2  rec2  6.4114687 1.24534089  
#6      sigma2_s      S  rec2  rec2 33.2148024 1.77130397  
#7    sigma2_lm1      S   lm1   lm1 71.7937576 6.18884357  
#8   sigma_g2lm1      S   lm1    g2 13.6858610 1.59341669  
#9     sigma2_g2      S    g2    g2  5.1477890 0.78835941  
#10 sigma_lr1lm1      S   lm1   lr1 56.4666648 5.18293807  
#11  sigma_lr1g2      S    g2   lr1 11.3788787 1.59736233  
#12   sigma2_lr1      S   lr1   lr1 71.7841065 7.33988029  
#13  sigma_j2lm1      S   lm1    j2  5.4954217 3.02238281 *
#14   sigma_j2g2      S    g2    j2  2.1198635 0.81358770 *
#15  sigma_j2lr1      S   lr1    j2 26.6014111 3.34179735  
#16    sigma2_j2      S    j2    j2 19.7654806 6.78819904  
#17    gamma_lm1      M     1   lm1 32.5775967 0.43127974  
#18     gamma_g2      M     1    g2 14.9401647 0.80669199  
#19    gamma_lr1      M     1   lr1 34.3629330 0.43401872  
#20     gamma_j2      M     1    j2 10.9038094 0.95866397  
#
#observed statistics:  4435 
#estimated parameters:  20 
#degrees of freedom:  4415 
#fit value ( -2lnL units ):  31364.88 
#number of observations:  933 
#Information Criteria: 
#      |  df Penalty  |  Parameters Penalty  |  Sample-Size Adjusted
#AIC:      22534.876               31404.88                       NA
#BIC:       1173.317               31501.64                 31438.13
#Some of your fit indices are missing.
#  To get them, fit saturated and independence models, and include them with
#  summary(yourModel, refModels=...) 
#  See help(mxRefModels) for an easy way of doing this in many cases. 
#timestamp: 2016-04-18 14:41:59 
#Wall clock time (HH:MM:SS.hh): 00:00:05.05 
#OpenMx version number: 2.2.4 
#Need help?  See help(mxSummary) 



#### Reading -> Changes in Mathematics ####
dcm.rm.omx <- mxModel("Reading to Mathematics Dual Change Model, Path Specification",
 type="RAM", mxData(observed=nlsy_math_wide, type="raw" ),
 manifestVars=c('math2','math3','math4','math5','math6','math7','math8',
                'rec2','rec3','rec4','rec5','rec6','rec7','rec8'),
 latentVars=c('lm1','lm2','lm3','lm4','lm5','lm6','lm7',
                    'dm2','dm3','dm4','dm5','dm6','dm7',
                    'g2',
              'lr1','lr2','lr3','lr4','lr5','lr6','lr7',
                    'dr2','dr3','dr4','dr5','dr6','dr7',
                    'j2'),

# defining true scores
mxPath(from=c('lm1','lm2','lm3','lm4','lm5','lm6','lm7',
              'lr1','lr2','lr3','lr4','lr5','lr6','lr7'),
 to=c('math2','math3','math4','math5','math6','math7','math8',
     'rec2','rec3','rec4','rec5','rec6','rec7','rec8'),
 arrows=1, free=FALSE, values=1),

# Autoregressive Effects
mxPath(from=c('lm1','lm2','lm3','lm4','lm5','lm6',
              'lr1','lr2','lr3','lr4','lr5','lr6'),
 to=c('lm2','lm3','lm4','lm5','lm6','lm7',
      'lr2','lr3','lr4','lr5','lr6','lr7'),
 arrows=1, free=FALSE, values=1),

# defining change scores
mxPath(from=c('dm2','dm3','dm4','dm5','dm6','dm7',
              'dr2','dr3','dr4','dr5','dr6','dr7'),
 to=c('lm2','lm3','lm4','lm5','lm6','lm7',
      'lr2','lr3','lr4','lr5','lr6','lr7'),
 arrows=1, free=FALSE, values=1),

# defining constant change factor for mathematics 
mxPath(from='g2', to=c('dm2','dm3','dm4','dm5','dm6','dm7'),
 arrows=1, free=FALSE, values=1),

# defining constant change factor for reading 
mxPath(from='j2', to=c('dr2','dr3','dr4','dr5','dr6','dr7'),
 arrows=1, free=FALSE, values=1),

# unique variance paths
mxPath(from=c('math2','math3','math4','math5','math6','math7','math8'),
 arrows=2, free=TRUE, values=35, labels='sigma2_u'),

mxPath(from=c('rec2','rec3','rec4','rec5','rec6','rec7','rec8'),
 arrows=2, free=TRUE, values=35, labels='sigma2_s'),

# unique covariance paths
mxPath(from=c('math2','math3','math4','math5','math6','math7','math8'),
 to=c('rec2','rec3','rec4','rec5','rec6','rec7','rec8'),
 arrows=2, free=TRUE, values=0, labels='sigma_su'),

# latent variable variances and covariances 
mxPath(from=c('lm1','g2','lr1','j2'), 
 arrows=2, connect = 'unique.pairs', free=TRUE, 
 values = c(75, 0, 0, 0, 7, 0, 0, 75, 0, 10),
 labels=c('sigma2_lm1','sigma_g2lm1','sigma_lr1lm1','sigma_j2lm1',
                       'sigma2_g2'  ,'sigma_lr1g2','sigma_j2g2',
                                     'sigma2_lr1' ,'sigma_j2lr1',
                                                   'sigma2_j2')),

# proportional parameters 
mxPath(from=c('lm1','lm2','lm3','lm4','lm5','lm6'),
 to=c('dm2','dm3','dm4','dm5','dm6','dm7'),
 arrows=1, free=TRUE, values=-.2, labels='pi_m'),

mxPath(from=c('lr1','lr2','lr3','lr4','lr5','lr6'),
 to=c('dr2','dr3','dr4','dr5','dr6','dr7'),
 arrows=1, free=TRUE, values=-.2, labels='pi_r'),

# coupling parameters 
mxPath(from=c('lr1','lr2','lr3','lr4','lr5','lr6'),
 to=c('dm2','dm3','dm4','dm5','dm6','dm7'),
 arrows=1, free=TRUE, values=0, labels='delta_m'),

# means and intercepts 
mxPath(from='one', to=c('lm1','g2','lr1','j2'),
 arrows=1, free=TRUE, values=c(30, 15, 30, 15),
 labels=c('gamma_lm1','gamma_g2','gamma_lr1','gamma_j2'))

) # close model

dcm.rm.fit <- mxRun(dcm.rm.omx)

summary(dcm.rm.fit) 

#Summary of Reading to Mathematics Dual Change Model, Path Specification 
# 
#free parameters:
#           name matrix   row   col   Estimate  Std.Error A
#1          pi_m      A   dm2   lm1 -0.4651689 0.12444373  
#2       delta_m      A   dm2   lr1  0.1931965 0.11011966  
#3          pi_r      A   dr2   lr1 -0.1525401 0.01633427  
#4      sigma2_u      S math2 math2 31.4888550 1.69795635  
#5      sigma_su      S math2  rec2  6.8856218 1.26999147  
#6      sigma2_s      S  rec2  rec2 33.8883986 1.83522635  
#7    sigma2_lm1      S   lm1   lm1 74.1993617 7.19938372  
#8   sigma_g2lm1      S   lm1    g2 16.8528363 2.34537455  
#9     sigma2_g2      S    g2    g2  9.3351927 3.36060381  
#10 sigma_lr1lm1      S   lm1   lr1 60.9133397 5.76766992  
#11  sigma_lr1g2      S    g2   lr1  6.5253912 4.10068422  
#12   sigma2_lr1      S   lr1   lr1 83.6079986 6.87973382  
#13  sigma_j2lm1      S   lm1    j2 11.0276991 1.75741727  
#14   sigma_j2g2      S    g2    j2  2.0921972 1.11296066  
#15  sigma_j2lr1      S   lr1    j2 18.0610989 1.92042147  
#16    sigma2_j2      S    j2    j2  5.3814484 0.94885073  
#17    gamma_lm1      M     1   lm1 32.2910753 0.45123914  
#18     gamma_g2      M     1    g2 16.1899436 0.86794935  
#19    gamma_lr1      M     1   lr1 34.1292913 0.43697627  
#20     gamma_j2      M     1    j2 12.3203479 0.77539545  
#
#observed statistics:  4435 
#estimated parameters:  20 
#degrees of freedom:  4415 
#fit value ( -2lnL units ):  31373.02 
#number of observations:  933 
#Information Criteria: 
#      |  df Penalty  |  Parameters Penalty  |  Sample-Size Adjusted
#AIC:      22543.022               31413.02                       NA
#BIC:       1181.463               31509.79                 31446.27
#Some of your fit indices are missing.
#  To get them, fit saturated and independence models, and include them with
#  summary(yourModel, refModels=...) 
#  See help(mxRefModels) for an easy way of doing this in many cases. 
#timestamp: 2016-04-18 14:43:53 
#Wall clock time (HH:MM:SS.hh): 00:00:05.36 
#OpenMx version number: 2.2.4 
#Need help?  See help(mxSummary) 



#### Full Coupling ####
dcm.fc.omx <- mxModel("Full Coupling Dual Change Model, Path Specification",
 type="RAM", mxData(observed=nlsy_math_wide, type="raw" ),
 manifestVars=c('math2','math3','math4','math5','math6','math7','math8',
                'rec2','rec3','rec4','rec5','rec6','rec7','rec8'),
 latentVars=c('lm1','lm2','lm3','lm4','lm5','lm6','lm7',
                    'dm2','dm3','dm4','dm5','dm6','dm7',
                    'g2',
              'lr1','lr2','lr3','lr4','lr5','lr6','lr7',
                    'dr2','dr3','dr4','dr5','dr6','dr7',
                    'j2'),

# defining true scores
mxPath(from=c('lm1','lm2','lm3','lm4','lm5','lm6','lm7',
              'lr1','lr2','lr3','lr4','lr5','lr6','lr7'),
 to=c('math2','math3','math4','math5','math6','math7','math8',
     'rec2','rec3','rec4','rec5','rec6','rec7','rec8'),
 arrows=1, free=FALSE, values=1),

# Autoregressive Effects
mxPath(from=c('lm1','lm2','lm3','lm4','lm5','lm6',
              'lr1','lr2','lr3','lr4','lr5','lr6'),
 to=c('lm2','lm3','lm4','lm5','lm6','lm7',
      'lr2','lr3','lr4','lr5','lr6','lr7'),
 arrows=1, free=FALSE, values=1),

# defining change scores
mxPath(from=c('dm2','dm3','dm4','dm5','dm6','dm7',
              'dr2','dr3','dr4','dr5','dr6','dr7'),
 to=c('lm2','lm3','lm4','lm5','lm6','lm7',
      'lr2','lr3','lr4','lr5','lr6','lr7'),
 arrows=1, free=FALSE, values=1),

# defining constant change factor for mathematics 
mxPath(from='g2', to=c('dm2','dm3','dm4','dm5','dm6','dm7'),
 arrows=1, free=FALSE, values=1),

# defining constant change factor for reading 
mxPath(from='j2', to=c('dr2','dr3','dr4','dr5','dr6','dr7'),
 arrows=1, free=FALSE, values=1),

# unique variance paths
mxPath(from=c('math2','math3','math4','math5','math6','math7','math8'),
 arrows=2, free=TRUE, values=35, labels='sigma2_u'),

mxPath(from=c('rec2','rec3','rec4','rec5','rec6','rec7','rec8'),
 arrows=2, free=TRUE, values=35, labels='sigma2_s'),

# unique covariance paths
mxPath(from=c('math2','math3','math4','math5','math6','math7','math8'),
 to=c('rec2','rec3','rec4','rec5','rec6','rec7','rec8'),
 arrows=2, free=TRUE, values=0, labels='sigma_su'),

# latent variable variances and covariances 
mxPath(from=c('lm1','g2','lr1','j2'), 
 arrows=2, connect = 'unique.pairs', free=TRUE, 
 values = c(75, 0, 0, 0, 7, 0, 0, 75, 0, 10),
 labels=c('sigma2_lm1','sigma_g2lm1','sigma_lr1lm1','sigma_j2lm1',
                       'sigma2_g2'  ,'sigma_lr1g2','sigma_j2g2',
                                     'sigma2_lr1' ,'sigma_j2lr1',
                                                   'sigma2_j2')),

# proportional parameters 
mxPath(from=c('lm1','lm2','lm3','lm4','lm5','lm6'),
 to=c('dm2','dm3','dm4','dm5','dm6','dm7'),
 arrows=1, free=TRUE, values=-.2, labels='pi_m'),

mxPath(from=c('lr1','lr2','lr3','lr4','lr5','lr6'),
 to=c('dr2','dr3','dr4','dr5','dr6','dr7'),
 arrows=1, free=TRUE, values=-.2, labels='pi_r'),

# coupling parameters 
mxPath(from=c('lr1','lr2','lr3','lr4','lr5','lr6'),
 to=c('dm2','dm3','dm4','dm5','dm6','dm7'),
 arrows=1, free=TRUE, values=0, labels='delta_m'),

mxPath(from=c('lm1','lm2','lm3','lm4','lm5','lm6'),
 to=c('dr2','dr3','dr4','dr5','dr6','dr7'),
 arrows=1, free=TRUE, values=0, labels='delta_r'),

# means and intercepts 
mxPath(from='one', to=c('lm1','g2','lr1','j2'),
 arrows=1, free=TRUE, values=c(30, 15, 30, 15),
 labels=c('gamma_lm1','gamma_g2','gamma_lr1','gamma_j2'))

) # close model

dcm.fc.fit <- mxRun(dcm.fc.omx)

summary(dcm.fc.fit) 

#free parameters:
          name matrix   row   col    Estimate Std.Error
1         pi_m      A   dm2   lm1 -0.29543517 0.1230187
2      delta_r      A   dr2   lm1  0.38055457 0.1342219
3      delta_m      A   dm2   lr1  0.05227485 0.1050620
4         pi_r      A   dr2   lr1 -0.48665444 0.1191467
5     sigma2_u      S math2 math2 31.43856473 1.6371060
6     sigma_su      S math2  rec2  6.46314035 1.2405521
7     sigma2_s      S  rec2  rec2 33.33472560 1.7852697
8   sigma2_lm1      S   lm1   lm1 72.74088290 6.6190641
9  sigma_g2lm1      S   lm1    g2 14.45852853 2.2332946
10   sigma2_g2      S    g2    g2  5.79127306 1.8144241
11 sigma_k1lm1      S   lm1   lr1 57.33893409 5.4944486
12 sigma_lr1g2      S    g2   lr1 10.03994085 3.1857454
13  sigma2_lr1      S   lr1   lr1 72.25983403 7.3402842
14 sigma_j2lm1      S   lm1    j2  6.29880756 3.1894127
15  sigma_j2g2      S    g2    j2  0.78975805 2.6251406
16 sigma_j2lr1      S   lr1    j2 25.97918917 3.4731049
17   sigma2_j2      S    j2    j2 17.84760947 6.8938650
18   gamma_lm1      M     1   lm1 32.52558939 0.4483195
19    gamma_g2      M     1    g2 15.20156973 0.9519057
20   gamma_lr1      M     1   lr1 34.36146206 0.4336121
21    gamma_j2      M     1    j2 10.98690301 0.9413406

observed statistics:  4435 
estimated parameters:  21 
degrees of freedom:  4414 
-2 log likelihood:  31364.6 
number of observations:  933 
Information Criteria: 
      |  df Penalty  |  Parameters Penalty  |  Sample-Size Adjusted
AIC:      22536.602               31406.60                       NA
BIC:       1179.881               31508.21                 31441.51














### Vector Field Plot based on Full Coupling Model ###

# Restructure data to long format #

nlsy_long_new = reshape(nlsy_math_wide, idvar='id', 
        varying=c('math2','rec2','math3','rec3','math4','rec4','math5','rec5',
                  'math6','rec6','math7','rec7','math8','rec8'),
        times=c(2,3,4,5,6,7,8),
        v.names=c('math','rec'), direction='long')

nlsy_long_new = nlsy_long_new[order(nlsy_long_new$id, nlsy_long_new$time),]

myvars = c('id','math','rec')

nlsy_long_new1 = nlsy_long_new[ ,myvars]

head(nlsy_long_new1)

## Calculating Means, Standard Deviations, and Correlation for Observed Variables ##
## Needed to Calculate Confidence Ellipse based on observed data ##

mean(nlsy_long_new1$math, na.rm=TRUE)
#46.11977
sd(nlsy_long_new1$math, na.rm=TRUE)
#12.79992
mean(nlsy_long_new1$rec, na.rm=TRUE)
#48.87263
sd(nlsy_long_new1$rec, na.rm=TRUE)
#15.06702
cor(nlsy_long_new1$math, nlsy_long_new1$rec, use="complete.obs")
#0.7297975


## Short Simulation to create originating and end-points of arrows ##

#y = math
#x = rec
df <- expand.grid(x=seq(10, 90, 5), y=seq(10, 90, 5))
#for changes in math
df$dy <- with(df, (15.201 - 0.295*y + 0.052*x)/5)
#for changes in reading
df$dx <- with(df, (10.987 - 0.487*x + 0.381*y)/5)
head(df)

library(ggplot2)
library(plotrix)
library(ellipse)
library(fields)

## Vector Field Plot in R ##

vf = ggplot(data=df, aes(x=x, y=y)) + geom_segment(aes(xend=x+dx, yend=y+dy), arrow = arrow(length = unit(0.3,"cm"))) +
            theme_bw() +
            scale_x_continuous(breaks = c(10,20,30,40,50,60,70,80,90), name = "Reading Recognition") + 
            scale_y_continuous(breaks = c(10,20,30,40,50,60,70,80,90), name = "Mathematics")
plot(vf)

ell = as.data.frame(ellipse(0.7297975, level=0.95, scale=c(15.06702,12.79992), centre=c(48.87263,46.11977), lwd=1.5))

vf_ellipse = vf + geom_path(data=ell)

plot(vf_ellipse)

